#! python3
import argparse
import os
from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientConnectorError
import asyncio
from .settings import DEFAULT_PORT, DESCRIPTION, ALIVE_SYMBOL
import urllib.parse
from colorama import Fore, Style
import subprocess
import sys
from .utils import printError
from time import sleep

def prompt():
    parser = argparse.ArgumentParser(
        description=DESCRIPTION)

    parser.add_argument("filePath",
                        nargs='*', metavar="file_Pame", type=str,
                        help="The file containing function to be exposed.")

    args = parser.parse_args()

    if not args.filePath:
        printError("No file path provided")
        return
    files = args.filePath

    targetServer = f'http://localhost:{DEFAULT_PORT}'

    async def main(reRun):
        async with ClientSession() as session:
            try:
                async with session.get(targetServer+'/isAlive') as resp:
                    if (resp.status == 200 and
                            (await resp.json()).get("res") == ALIVE_SYMBOL["python"]):
                        for fileName in files:
                            params = urllib.parse.urlencode({
                                "filePath": fileName,
                                "workingPath": os.getcwd()
                            })
                            async with session.get(
                                f'{targetServer}/funcRegister?{params}'
                            ) as resp:
                                funcName = await resp.json()
                                if funcName['status'] == "ERR":
                                    printError(funcName['res'])
                                else:
                                    print(f"{Fore.BLUE}{funcName['res']}{Style.RESET_ALL} registered on {Fore.RED}{targetServer}{Style.RESET_ALL}")
                    else:
                        printError("target server is not working properly or is not gbcall server")
            except ClientConnectorError:
                t=subprocess.Popen([sys.executable, os.path.split(os.path.realpath(__file__))[0]+'/server.py'],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.STDOUT)
                if not reRun:
                    sleep(0.5) #TODO: 改成读到启动再重试
                    await main(True)
                else:
                    printError("start server failed")


    asyncio.run(main(False))



import re
from colorama import Fore, Style


import importlib.util
import sys

def scanner(code):
    regex = re.compile("@defineExpose\((.+)\)\ndef (.+):", re.S)
    IOStr = re.search(regex, code).group(1)
    SignStr = re.search(regex, code).group(2)
    args = re.search(re.compile(
        "input=\[(.*?)\]", re.S), IOStr).group(1).split(',')
    args = list(map(lambda s: s.strip(), args))
    args = list(filter(lambda s: s != '', args))
    ret = re.search(re.compile("output=\[(.*?)\]", re.S), IOStr).group(1).split(',')
    ret = list(map(lambda s: s.strip(), ret))
    ret = list(filter(lambda s: s != '', ret))
    funcName = re.search(re.compile("(.+?)\(.+?\)", re.S), SignStr).group(1)
    return funcName, args, ret


def mapAppend(m, key, val):
    if key in m:
        m.append(val)
    else:
        m[key] = [val]


def printError(txt):
    print(f"{Fore.RED}ERROR:{Style.RESET_ALL} {txt}")

def dynamicImport(path):
    spec = importlib.util.spec_from_file_location(
        path, path)
    foo = importlib.util.module_from_spec(spec)
    sys.modules[path] = foo
    spec.loader.exec_module(foo)
    return foo



from .typeConverter import convertParameters,typeNameConvert
def defineExpose(input, output):
    def decorator(func):
        def wrapper(*args):
            inBrowser=False
            try:
                __file__
            except:
                inBrowser=True
            if inBrowser:
                inputTypes=list(map(typeNameConvert,input))
                outputTypes=list(map(typeNameConvert,output))
                return convertParameters(outputTypes,func(*convertParameters(inputTypes,args)))
            return func(*args)
        return wrapper
    return decorator



from setuptools import setup

VERSION = '0.0.8'

setup(
    name='global-call',  # package name
    version=VERSION,  # package version
    description='A simple serverless function transformer,\
     and a simple way to call function in other language.',  # package description
    packages=['gbcall'],
    # package_dir={"": "src"},
    zip_safe=False,
    author="bi1lqv",
    author_email="bi1lqy.y@gmail.com",
    license="MIT",
    install_requires=[
        "aiohttp",
        "mpld3"
    ],
    entry_points={
        'console_scripts': [
            'gbcall = gbcall.cli:prompt',
            'gbcall-server = gbcall.server:run',
        ]
    }
)



from . import types
import re

def __number(maybeNumber):
  return float(maybeNumber)

def __text(maybeText):
  return str(maybeText)

def __boolean(maybeBoolean):
  return bool(maybeBoolean)

def __null(maybeNull):
  return None

def __plot(maybePlot):
  import mpld3
  return  mpld3.fig_to_dict(maybePlot)

def __complex(val):
  return val

typeConverter={
  "Number":__number,
  "Text":__text,
  "Boolean":__boolean,
  "Null":__null,
  "Plot":__plot,
  "Complex":__complex
}

def convertParameters(types,parameters):
  if len(types)==1 and not isinstance(parameters,list):
    parameters=[parameters]
  try:
    return list(map(lambda type,val:typeConverter[type](val),types,parameters))
  except Exception:
    raise Exception("unknown type")

allTypes=list(filter(lambda name:not name.startswith("__"),dir(types)))

def typeNameConvert(typeName):
    for type in allTypes:
        if re.compile(f".*{type}$").match(typeName):
            return type
    print(typeName)
    raise Exception("unknown type")